@@ -11,7 +11,7 @@ import starboard
 
 from .. import wait
 from ..users import User
-from ..machines import MachineWrapper
+from ..machines import MachineWrapper, MachineSet
 from .. import processes
 from .. import dictobj
 from ..request import request_machine, MachineRequest
@@ -295,30 +295,6 @@ MachineStatus = dictobj.data_class("MachineStatus",
 )
 
 
-class MachineSet(object):
-    def __init__(self, machines):
-        self._machines = machines
-        
-    def __enter__(self):
-        return self
-        
-    def __exit__(self, *args):
-        for machine in self._machines:
-            machine.destroy()
-            
-    def __getitem__(self, key):
-        if isinstance(key, basestring):
-            return next(
-                machine
-                for machine in self._machines
-                if machine.name == key
-            )
-        elif isinstance(key, (int, long)):
-            return self._machines[key]
-        else:
-            raise TypeError("Expected key to be string or integer")
-
-
 class Statuses(object):
     def __init__(self, status_dir):
         self._status_dir = status_dir
@@ -371,8 +347,7 @@ class Statuses(object):
     def _write_json(self, identifier, data):
         status_path = self._status_path(identifier)
         
-        if not os.path.exists(os.path.dirname(status_path)):
-            os.makedirs(os.path.dirname(status_path))
+        _mkdir_p(os.path.dirname(status_path))
             
         with open(status_path, "w") as status_file:
             json.dump(data, status_file)
@@ -407,14 +382,14 @@ class QemuMachine(object):
         
         self._statuses.remove(self.identifier)
         
-    def public_port(self, guest_port):
-        return self._forwarded_ports.get(guest_port, None)
-        
     def external_hostname(self):
         return starboard.find_local_hostname()
         
     def users(self):
         return self._users
+    
+    def forwarded_tcp_ports(self):
+        return self._forwarded_ports
 
 
 class UserNetworking(object):
@@ -470,3 +445,11 @@ def _generate_forwarded_ports(image, public_ports):
     public_ports = set([image.ssh_internal_port] + public_ports)
     host_ports = starboard.find_local_free_tcp_ports(len(public_ports))
     return dict(zip(public_ports, host_ports))
+
+
+def _mkdir_p(path):
+    try:
+        os.makedirs(path)
+    except OSError as error:
+        if not (error.errno == errno.EEXIST and os.path.isdir(path)):
+            raise
